# -*- coding: iso-8859-1 -*-

TXEnableDelay = 2e-6
TXEnableValue = 0b0001		# TTL line blanking RF amplifier (bit 0)
TXPulseValue  = 0b0010		# TTL line triggering RF pulses  (bit 1)
ADCSensitivity = 2		# voltage span for ADC

def experiment(): # fid aquisition with background signal suppresion (the Bruker's zgbs)

  # set up acqusition parameters:      
    pars = {}
    pars['P90'] = 3.5e-6	# 90-degree pulse length (s)
    pars['SF'] = 360.0e6	# spectrometer frequency (Hz)
    pars['O1'] = -140e3  	# offset from SF (Hz)
    pars['SW'] = 10e6		# spectrum width (Hz)
    pars['SI'] = 4*1024		# number of acquisition points
    pars['NS'] = 64             # number of scans
    pars['DS'] = 0              # number of dummy scans
    pars['RD'] = 1              # delay between scans (s)
    pars['DEAD1'] = 5e-6	# receiver dead time (s)  
    pars['PHA'] = 0             # receiver phase (degree)
    pars['DATADIR'] = '/home/fprak/Students/'	# data directory
    pars['OUTFILE'] = None                     	# output file name

  # specify a variable parameter (optional):
    pars['VAR_PAR'] = None	# variable parameter name (a string)   
    start = 1e-6           	# starting value
    stop = 5e-6            	# end value
    steps = 5              	# number of values
    log_scale = False		# log scale flag 
    stag_range = False		# staggered range flag

  # check parameters for safety:     
    if pars['PHA'] < 0:
        pars['PHA'] = 360 + pars['PHA']

    if pars['P90'] > 20e-6:
        raise Exception("Pulse too long!!!")
                    
    # check whether a variable parameter is named: 
    var_key = pars.get('VAR_PAR')
    if var_key == 'P90' and (start > 20e-6 or stop > 20e-6):
        raise Exception("Pulse too long!!!")

    if pars['NS']%16 != 0:
        pars['NS'] = int(round(pars['NS'] / 16) + 1) * 16 
        print 'Number of scans changed to ', pars['NS'], ' due to phase cycling'

    # start the experiment:
    if var_key: 
        # this is an arrayed experiment:
        if log_scale:
            array = log_range(start,stop,steps)
        else:
            array = lin_range(start,stop,steps) 

        if stag_range:
            array = staggered_range(array, size = 2)

        # estimate the experiment time:
        if var_key == 'RD':
            seconds = sum(array) * (pars['NS'] + pars['DS'])
        else:
            seconds = pars['RD'] * steps * (pars['NS']+ pars['DS'])
        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        print '%s%02d:%02d:%02d' % ('Experiment time estimated: ', h, m, s)
 
        # loop for a variable parameter:
        for index, pars[var_key] in enumerate(array):
            print 'Arrayed experiment for '+var_key+': run = '+str(index+1)+\
                             ' out of '+str(array.size)+': value = '+str(pars[var_key])
            # loop for accumulation:
            for run in xrange(pars['NS']+pars['DS']):
                yield zgbs_experiment(pars, run) 
            synchronize()
    else:
        # estimate the experiment time:
        seconds = pars['RD'] * (pars['NS']+ pars['DS'])
        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        print '%s%02d:%02d:%02d' % ('Experiment time estimated: ', h, m, s)

        # loop for accumulation:
        for run in xrange(pars['NS']+pars['DS']):
            yield zgbs_experiment(pars, run)


# the pulse program: 

def zgbs_experiment(pars, run):
    e=Experiment()

    dummy_scans = pars.get('DS')
    if dummy_scans:
        run -= dummy_scans

    pars['PROG'] = 'zgbs_experiment'

    # phase lists [from Bruker's zgbs]:
    pars['PH1'] = [0, 0, 0, 0, 90, 90, 90, 90, 180, 180, 180, 180, 270, 270, 270, 270]	# 90-degree pulse
    pars['PH3'] = [0, 90, 180, 270]							# 180-degree pulse
    pars['PH4'] = [0, 0, 0, 0, 180, 180, 180, 180, 270, 270, 270, 270, 90, 90, 90, 90]	# 180-degree pulse
    pars['PH2'] = [0, 180, 0, 180, 90, 270, 90, 270]					# receiver

    # read in variables:
    P90 = pars['P90'] 
    P180 = 2*pars['P90']
    SF = pars['SF']
    O1 = pars['O1']   
    RD = pars['RD']
    DEAD1 = pars['DEAD1'] 
    PH1 = pars['PH1'][run%len(pars['PH1'])]
    PH2 = pars['PH2'][run%len(pars['PH2'])]
    PH3 = pars['PH3'][run%len(pars['PH3'])]
    PH4 = pars['PH4'][run%len(pars['PH4'])]
    PHA = pars['PHA']

    # set sampling parameters:
    SI = pars['SI']
    SW = pars['SW']
    while SW <= 10e6 and SI < 256*1024:
        SI *= 2
        SW *= 2

    # run the pulse sequence: 
    e.wait(RD)                                          # delay between scans
    e.set_frequency(SF+O1, phase=PH1)                   # set frequency and phase for RF pulse
    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)	# enable RF amplifier
    e.ttl_pulse(P90, value=TXEnableValue|TXPulseValue)	# apply 90-degree pulse
    e.set_phase(PH3) 
    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)	# enable RF amplifier
    e.ttl_pulse(P180, value=TXEnableValue|TXPulseValue)	# apply 180-degree pulse
    e.set_phase(PH4)    
    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)	# enable RF amplifier
    e.ttl_pulse(P180, value=TXEnableValue|TXPulseValue)	# apply 180-degree pulse
    e.set_phase(PHA)					# set phase for receiver 
    e.wait(DEAD1)					# wait for coil ringdown
    e.record(SI, SW, sensitivity=ADCSensitivity)	# acquire signal

    # write experiment parameters:  
    for key in pars.keys():
        e.set_description(key, pars[key])		# acqusition parameters
    e.set_description('run', run)			# current scan
    e.set_description('rec_phase', -PH2)		# current receiver phase

    return e